/****************************************
 * Πρόγραμμα που μετρά το χρόνο που πέρασε
 * μεταξύ της πτώσης του παλμού στον ακροδέκτη 2
 * και της πτώσης του παλμού στον ακροδέκτη 3 που
 * συνδέονται δύο αισθητήρες KY-002.
 * 
 * Όταν ανιχνευτεί πτώση του παλμού στον ακροδέκτη
 * 2 τότε ξεκινά η μέτρηση του χρόνου και όταν 
 * ανιχνευτεί πτώση του παλμού στον ακροδέκτη 3
 * σταματά η μέτρηση του χρόνου και το αποτέλεσμα
 * αποθηκεύεται στη μεταβλητή ElapsedTime ενώ
 * τείθεται σε λογικό 1 η μεταβλητή TimeReady
 */

// Δήλωση μεταβλητών
static volatile boolean TimeReady = false;
static volatile unsigned long TimeCaptureStart = 0;
static volatile unsigned long TimeCaptureEnd = 0;
unsigned long ElapsedTime = 0;

void setup() 
{
  // Ρύθμιση των ακροδεκτών σε είσοδο
  pinMode(2,INPUT);
  pinMode(3,INPUT);
  // προσάπτουμε μία διακοπή σε κάθε ακροδέκτη
  // και ορίζουμε την ρουτινα εξυπηρέτησης της
  // διακοπής που θα εκτελείται κατά την πτώση
  // του παλμού στον κάθε ακροδέκτη
  attachInterrupt(digitalPinToInterrupt(2),KY_002_1_Interrupt,FALLING);
  attachInterrupt(digitalPinToInterrupt(3),KY_002_2_Interrupt,FALLING);
  // ρύθμισε τη σειριακή θύρα για να εκτυπώνουμε τα αποτελέσματα στο 
  // τερματικό
  Serial.begin(9600);
}

void loop() 
{
  // όταν ο χρόνος μεταξύ των δύο κρούσεων έχει μετρηθεί
  if (TimeReady)
  {
    // κάνε υπολογισμό του χρόνου που πέρασε
    ElapsedTime = TimeCaptureEnd - TimeCaptureStart;
    // εμφάνισε στη σειριακή θύρα το αποτέλεσμα σε ms
    Serial.print("Elapsed Time = ");
    Serial.println(ElapsedTime);      
    // μηδένισε τη μεταβλητή ώστε να μήν εκτελεστεί ξανά η διαδικασία
    TimeReady = false;

    // Μετά απο αυτό το σημείο προκειμένου να κάνουμε νέα μέτρηση
    // πρέπει να πατήσουμε το reset στη μονάδα Arduino Uno και να
    // ξεκινήσουμε μία νέα μέτρηση απο την αρχή τιατί έχουμε
    // απενεργοποιήσει τις διακοπές στους ακροδέκτες 2 και 3  
}

// Ρουτίνες εξυπηρέτησης διακοπών 
// Ρουτίνα για την διακοπή κατά την πτώση του παλμού
// στον ακροδέκτη 2
void KY_002_1_Interrupt()
{
  // όταν πραγματοποιηθεί διακοπή στον ακροδέκτη 2
  // σημαίνει έναρξη της μέτρησης του χρόνου  
  // αντέγραψε τον χρόνο αυτή τη στιγμή
  TimeCaptureStart = millis();  
  // απενεργοποίησε τη διακοπή απο τον ακροδέκτη 2
  // για να μήν δημιουργηθεί φαινόμεο αναπήδησης
  // θέλουμε να εκτελεστεί αυτή η ρουτίνα μόνο 1 φορά
  detachInterrupt(digitalPinToInterrupt(2));
  TimeReady = false;
}
// Ρουτίνα για την διακοπή κατά την πτώση του παλμού
// στον ακροδέκτη 3
void KY_002_2_Interrupt()
{
  // όταν πραγματοποιηθεί διακοπή στον ακροδέκτη 3
  // σημαίνει λήξη της μέτρησης του χρόνου  
  // αντέγραψε τον χρόνο αυτή τη στιγμή  
  TimeCaptureEnd = millis();   
  // δήλωσε ότι ο χρόνος μετρήθηκε
  TimeReady = true;
  // απενεργοποίησε τη διακοπή απο τον ακροδέκτη 3
  // για να μήν δημιουργηθεί φαινόμεο αναπήδησης
  // θέλουμε να εκτελεστεί αυτή η ρουτίνα μόνο 1 φορά
  detachInterrupt(digitalPinToInterrupt(3)); 
}
