/******************************************************************************
*  Πρόγραμμα εφαρμογής Web Server με τη μονάδα Arduino Uno και τη μονάδα Arduino Ethernet
*  shield.
*   *****************************************************************************/
// Συμπεριλαμβάνουμε τη βιβλιοθήκη οδήγησης SPI για την οδήγηση του ελεγκτή Ethernet
#include <SPI.h>
// Συμπεριλαμβάνουμε τη βιβλιοθήκη Ethernet ώστε να χρησιμοποιήσουμε τη στοίβα του 
// του πρωτοκόλλου Ethernet και όλες τις μεθόδους του.
#include <Ethernet.h>

// Κάθε συσκευή σε ένα δίκτυο Ethernet έχει μία μοναδική διεύθυνση MAC, ορίζουμε αυτή την 
// διεύθυνση
byte mac[] = { 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF };
// Κάθε συσκευή σε ένα δίκτυο Ethernet έχει μία μοναδική διεύθυνση IP στο τοπικό δίκτυο 
IPAddress ip(192,168,1,10);

// Δηλώνουμε ένα αντικείμενο της κλάσης Ethernet χρησιμοποιώντας την κατάλληλη θύρα 
// επικοινωνίας για να έχουμε πρόσβαση στις μεθόδους της στοίβας Ethernet 
EthernetServer server(80);

void setup() 
{
  // Κάνουμε έναρξη της χρήσης υπηρεσιών Ethernet για την θύρα επικοινωνίας και την IP διεύθυνση
  // του δικτύου μας
  Ethernet.begin(mac, ip);
  // κάνουμε έναρξη των υπηρεσιών του εξυπηρετητή (server) 
  server.begin();
}

void loop() 
{
  // σε κάθε επανάληψη κάνε έλεγχο αν υπάρχουν Clients που επιθυμούν να επισκεφθούν την
  // ιστοσελίδα μας. Κάθε Client θα στέλνει ένα αίτημα HTTP προς τον εξυπηρετητή.
  EthernetClient client = server.available();
  if (client) 
  {    
    // ένα αίτημα http τελειώνει με μία κενή γραμμή επομένως θέσε τη μεταβλητή σε true
    boolean currentLineIsBlank = true;
    // για όση ώρα ο client βρίσκεται συνδεδεμένος με τον εξυπηρετητή.
    while (client.connected()) 
    {
      // αν ο client είναι ενεργός
      if (client.available()) 
     {
        // κάνε ανάγνωση των δεδομένων του αιτήματος http που στέλνει ο Client.
        char c = client.read();
        // Αν έχουμε φτάσει στο τέλος της γραμμής ανιχνεύοντας ένα τερματικό χαρακτήρα 
         //  νέας γραμμής \n και αν η γραμμή είναι κενή τότε αυτό σημαίνει πως το αίτημα http 
         //   ολοκληρώθηκε
        if (c == '\n' && currentLineIsBlank) 
        {
          // σε αυτό το σημείο ο εξυπηρετητής θα στείλει μία απάντηση http στον Client.
           // που είναι κώδικας HTML ώστε να εμφανιστεί στον Client η ιστοσελίδα μας
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");
          client.println();
          client.println("<!DOCTYPE HTML>");
          client.println("<html>");
          // τοποθετούμε ένα tag στον κώδικα HTML ώστε ο browser του client να ανανεώνει τη σελίδα
           // κάθε 10 δευτερόλεπτα.
          client.println("<meta http-equiv=\"refresh\" content=\"10\">");
          // Σε αυτό το σημείο θα κάνουμε ανάγνωση των δεδομένων του αισθητήρα θερμοκρασίας στο
           // αναλογικό κανάλι 0 και θα υπολογίσουμε την τιμή της θερμοκρασίας
          int adc_value = analogRead(A0);
          // υπολόγισε την τιμή της θερμοκρασίας σε βαθμούς C
            float temperature = ((adc_value * 5.0f) / 1024) / 10; 
            client.print("Temperature = ");
            client.print(temperature);
            client.print(" C ");            
            client.println("<br />");       
          }
          client.println("</html>");
          break;
        }
        // αν ο χαρακτήρας που αναγνώσαμε είναι ο χαρακτήρας αλλαγής γραμμής
        if (c == '\n') 
        {
          // θέσε την τιμή της μεταβλητής σε true για να δηλώσουμε ότι ξεκινάμε μία νέα γραμμή
          currentLineIsBlank = true;
        } 
        else if (c != '\r')  // αλλιώς αν ΔΕΝ είναι ο χαρακτήρας επιστροφής (return)
        {
          // θέσε τη μεταβλητή σε false για να δηλώσουμε ότι λάβαμε έναν χαρακτήρα του αιτήματος
          currentLineIsBlank = false;
        }
      }
    }
    // κάνε μία μικρή καθυστέρηση της τάξης του 1 ms ώστε ο browser να λάβει τα δεδομένα
    delay(1);
    // κλείνουμε την σύνδεση με τον Client
    client.stop();
  }
}
