/**********************************************************************
*  Πρόγραμμα οδήγησης lcd οθόνης 16x2 (HD44780)
*  με την αναπτυξιακή μονάδα Arduino UNO για απεικόνιση δεδομένων θερμοκρασίας
*  του αισθητήρα LM35 τα δεδομένα του οποίο λαμβάνονται ασύρματα μέσω συσκευών
*  XBEE
************************************************************************/
// Συμπεριλαμβάνουμε την βιβλιοθήκη οδήγησης της οθόνης LCD
#include <LiquidCrystal.h>

// Ορισμός των ακροδεκτών οδήγησης της οθόνης
int LCD_RS  = 2 ;
int LCD_E  =  3;
int LCD_D4  =  4;
int LCD_D5  =  5;
int LCD_D6  =  6;
int LCD_D7  =  7;
ind LCD_LED = 8;

// Δημιουργούμε ένα αντικείμενο της κλάσης LiquidCrystal με όνομα lcd 
// και θέτουμε στις παραμέτρους τους ακροδέκτες που έχουμε ορίσει 
// για την οδήγηση της οθόνης σύμφωνα με το κυκλωματικό διάγραμμα
LiquidCrystal lcd (LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

// μεταβλητή που θα κρατά τα δεδομένα που έρχονται από τη σειριακή θύρα UART
String serialBuffer = "";
// μεταβλητή που θα δηλώνει αν ολοκληρώθηκε η λήψη δεδομένων στη σειριακή θύρα UART
bool receiveComplete = false;

void setup(void)
{
  // Ρυθμίζουμε την οθόνη LCD ορίζοντας τον αριθμό των χαρακτήρων και των γραμμών
  // που αυτή υποστηρίζει καλώντας την ιδιότητα begin του αντικειμένου lcd που δημιουργήσαμε.
  // Στο παράδειγμά μας έχουμε οθόνη 16 χαρακτήρων και 2 γραμμών
  lcd.begin (16, 2);

       // κάνουμε χρήση της σειριακής επικοινωνίας UART με BaudRate 9600
       // το Baud Rate θα πρέπει να είναι ίδιο και στη μονάδα XBEE
       Serial.begin(9600);
      // δέσμευσε χώρο για τα δεδομένα του καταχωρητή λήψης UART
       serialBuffer.reserve(200);
}

void loop()
{    
          // σε κάθε επανάληψη κάνε έλεγχο αν έχουμε λάβει δεδομένα από τη
         // μονάδα UART που σημαίνει πως τα δεδομένα έφτασαν ασύρματα
         // από την απομακρυσμένη συσκευή XBEE που μετρά την θερμοκρασία 
         if (receiveComplete)
         {
            // τα δεδομένα είναι σε μορφή συμβολοσειράς μέσα στη μεταβλητή 
            // serialBuffer. Θα μετατρέψουμε τα δεδομένα σε μορφή byte 
            char packet_data[serialBuffer.length()];
            // θα τα τοποθετήσουμε στον πίνακα packet_data
            cmdString.toCharArray(packet_data, serialBuffer.length()) ;
            // το πρώτο byte των δεδομένων αντιστοιχεί στο LSB της θερμοκρασίας
            int adc_lsb = packet_data[0];
            // το δεύτερο byte των δεδομένων αντιστοιχεί στο MSB της θερμοκρασίας
            int adc_msb = packet_data[1];
            // κάνουμε σύνδεση των 2 byte για να πάρουμε την τιμή της θερμοκρασίας
            // που διάβασε ο αισθητήρας
            int adc_value = adc_msb << 8;
            adc_value = adc_value | adc_lsb;
            // υπολογίζουμε την τιμή της θερμοκρασίας σε βαθμούς C από την τιμή του μετατροπέα adc
            float temperature = ( (adc_value * 5.0f )/1024) / 10;
            // Εμφάνισε την τιμή της θερμοκρασίας στην οθόνη LCD
            lcd.print ("Temp = ");
            lcd.print (temperature,DEC);
            lcd.print (" C");
            // μηδένισε τη μεταβλητή για επόμενο έλεγχο λήψης
            receiveComplete = false;
            // καθάρισε μεταβλητή για επόμενο έλεγχο λήψης
            serialBuffer = "";
         }
}

// εκτελείται σε κάθε επανάληψη σύμφωνα με το reference της πλατφόρμας
// arduino
void serialEvent()
{
   while(Serial.available())
   {
      // κάνε ανάγνωση του χαρακτήρα που ήρθε στη σειριακή θύρα
      char inputChar = (char)Serial.read();
      // αν ο χαρακτήρας που αναγνώσαμε είναι ο τερματικός χαρακτήρας 
      if (inputChar == '\n')
            // δήλωσε πως η λήψη ολοκληρώθηκε
            receiveComplete = true;
   }
}
